<?php

namespace RotinaEstoque\Controllers\Planning;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use \Setforms2\Shared\ORM\DateTime;

class WithdrawalPlanningController {
  protected $crudService;
  const COD_ERROR_PEBBIAN = 1500;
  const PREVRETI = 'withdrawalPredict';
  
  protected $dataSourceName = "withdrawalPlanning";
  protected $parameterBagColumns = array(
    "NRORG",
		"CDFILIAL",
		"CDSERVFILI",
		"DTRETIRADA",
		"DTCARDAPIO",
		"NRREQUEXTRRE"
	);

  public function __construct($connection, $environment, $util, $service, $crudService, $libUtilities, $dataSourceManager) {
    $this->connection        = $connection;
    $this->environment       = $environment;
    $this->util              = $util;
    $this->service           = $service;
    $this->crudService	     = $crudService;
    $this->libUtilities      = $libUtilities;
    $this->dataSourceManager = $dataSourceManager;
  }
  
  private function treatError(\Exception $e, $response){
    $message = $e->getMessage();
    $code = $e->getCode();
    $arrayMessage = explode("\n", $message);
    if($this->isExceptionPebbian($e)){
      $response->setError(new DTO\Response\Error($message, $code));
      return;
    }
    if(!empty($arrayMessage)){
      $clean_message = "";
      foreach($arrayMessage as $currentString){
        if(stripos($currentString, "ORA-20501") !== false){
          $clean_message = str_replace("ORA-20501: ", "", $currentString);
        }
      }
      if(!empty($clean_message)) {
        $response->setError(new DTO\Response\Error(utf8_decode($clean_message), 1)); 
      } else {
        $response->setError(new DTO\Response\Error($e->getMessage(), 1));
      }
    } else {
      $response->setError(new DTO\Response\Error($e->getMessage(), 1));
    }
  }

  private function isExceptionPebbian(\Exception $error){
    return $error->getCode() === $this->service->getCodErrorPebbian();
  }

  public function saveWithdrawal(Request\Row $request, Response $response) {
    try {
      $row = $request->getRow();
      $cdservfili = $row['CDSERVFILI'];
      $dtretirada = $row['DTRETIRADA'];
      $cdtpretipadr = $row['CDTPRETIPADR'];
      $cdfilial_logada = $this->environment->getCdfilial();
      
      // coleta de dados
      $this->libUtilities->saveCollectedData($row['CDFILIAL'],'RETIRADA');
      
      $this->setParamsEstoq($row);
      $retirada = self::retornaRetirada($row);
      if (!empty($retirada)) {
        throw new \Exception("Já existe retirada cadastrada.");
      }

      $validaDataRetirada = $this->validaDataRetirada($row);

      if ($validaDataRetirada) {
        throw new \Exception($validaDataRetirada);
      }

      if (!$cdtpretipadr) {
        throw new \Exception("Não existe tipo de retirada parametrizado.");
      }
      $row['DTPREVRETI'] = $row['DTCARDAPIO'];
      $prevreti = $this->retornaPrevreti($row);

      if($prevreti) {
      	$idretiprat = $prevreti[0]['IDPREVTPPRAT'];
      	$nrcardapio = $this->service->retornaNrcardapio($row);
      	if(!$nrcardapio) {
      		$nrcardapio = $this->service->insereCardapio($row);
      	}
      	$row['IDTPLANCTO'] = 3;
      	$row['DTLANCESTQ'] = $dtretirada;
      	$row['DSLANCESTQ'] = "Retirada para o cardápio: ".$dtretirada.", Servico: ".$cdservfili;
      	$row['NRCARDAPIO'] = $nrcardapio;
        $row['IDUTRETIPRAT'] = $idretiprat;
      	
      	$nrlancto = $this->service->insereLancamento($row);
        $row['NRLANCESTQ'] = $nrlancto;
          
      	$nrretirada = $this->service->insereRetirada($row);

        $index = 0;
      	foreach ($prevreti as $item) {
          $seq = str_pad($index++, 4, "0", STR_PAD_LEFT);
          $item['NRLANCESTQ'] = $nrlancto;
          $item['NRSEQUITEM'] = $seq;
          $item['QTRETIRADA'] = $item['QTPREVLIB'];
          $item['IDTIPOMOVI'] = '3';
          $item['DTRETIRADA'] = $row['DTRETIRADA'];
          $item['IDORDEMCLC'] = 9;
          $item['CDFILIAL'] = $row['CDFILIAL'];
          $item['CDFILIMOVI'] = $row['CDFILIMOVI'];
          $item['IDEVOLESTOQ'] = 'N';
          $item['CDTIPORETI'] = $row['CDTPRETIPADR'];
          $item['NRRETIRADA'] = $nrretirada;
          $item['NRCARDAPIO'] = $nrcardapio;
          $item['IDINTESIST'] = 'N';
          $item['QTPREVISRETI'] = $item['QTPREVRETI'];
          $item['QTDEVOLVRETI'] = $item['QTPREVDEVOL'];
          $item['QTEXTRARETI'] = $item['QTPREVEXTRA'];
          $this->setParamsEstoq($item);
  			  $this->service->insereItemLancto($item, $cdfilial_logada);
      		$this->service->insereItemreti($item, $cdfilial_logada );
      	}
      	$response->addMessage(new Message(true));
      } else {
      	$response->addMessage(new Message(false));
      }

    } catch (\Exception $e) {
        $response->setError(new DTO\Response\Error($e->getMessage(), 1));
      }
  }
  
  public function retornaPrevreti($row) {
		$filterCriteria = new FilterCriteria(self::PREVRETI);
		$this->parameterBagColumns = array('NRORG', 'CDFILIAL', 'CDSERVFILI', 'DTPREVRETI', 'NRREQUEXTRRE');
		$filterCriteria->addCondition('NRORG', $this->environment->getNrOrg());
		$filterCriteria->addCondition('CDFILIAL', $row['CDFILIAL']);
		$filterCriteria->addCondition('CDSERVFILI', $row['CDSERVFILI']);
		$filterCriteria->addCondition('DTPREVRETI', $row['DTPREVRETI']);
		$filterCriteria->addCondition('NRREQUEXTRRE', $row['NRREQUEXTRRE']);
		$newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);
  	$dataSet = $this->dataSourceManager->findBy($newFilterCriteria);
  	$rows = $dataSet->getRows();

  	return $rows;
  }

  public function validaDataRetirada($row) {
    $mensagem = false;
    $dtprocessa = \DateTime::createFromFormat('d/m/Y H:i:s', $row['DTPROCESSA']);
    $dtretirada = \DateTime::createFromFormat('d/m/Y H:i:s', $row['DTRETIRADA']."00:00:00");
    $idlancdtmen = $row['IDLANCDTMEN'];
    $idlancdtmai = $row['IDLANCDTMAI'];
    $nrdiassaid = $row['NRDIASSAID'] ? $row['NRDIASSAID'] : 0;
    $nrdiassaidpd = $row['NRDIASSAIDPD'] ? $row['NRDIASSAIDPD'] : 0;
    $idlancdtmenfil = $row['IDLANCDTMENFIL'];
    $nrdiassaidfil = $row['NRDIASSAIDFIL'] ? $row['NRDIASSAIDFIL'] : 0;
    $idlancdtmaifil = $row['IDLANCDTMAIFIL'];
    $nrdiassaidpdfil = $row['NRDIASSAIDPDFIL'] ? $row['NRDIASSAIDPDFIL'] : 0;
    $dateProcessa = \DateTime::createFromFormat('d/m/Y', explode(" ",$row['DTPROCESSA'])[0]);
    $dateRetirada = \DateTime::createFromFormat('d/m/Y', $row['DTRETIRADA']);
    $dtprocessaformat = strtotime($dateProcessa->format('Y-m-d'));
    $dtretiradaformat = strtotime($dateRetirada->format('Y-m-d'));
    if ($dtretiradaformat < $dtprocessaformat) {
      if ($idlancdtmenfil == 'N') {
        if ($idlancdtmen == 'N') {
          $mensagem = "Data informada não deve ser menor que a data de processamento.";
        } else {
          if( $dtretirada < ($dtprocessa->sub(new \DateInterval('P'.$nrdiassaid.'D')))) {
            $mensagem = "Data informada fora do período permitido.";
          }
        }
      } else {
        if($dtretirada < ($dtprocessa->sub(new \DateInterval('P'.$nrdiassaidfil.'D')))) {
          $mensagem = "Data informada fora do período permitido.";
        }
      }
    } else if ($dtretiradaformat > $dtprocessaformat) {
      if ($idlancdtmaifil == 'S') {
        if( $dtretirada > ($dtprocessa->add(new \DateInterval('P'.$nrdiassaidpdfil.'D')))) {
          $mensagem = "Data informada fora do período permitido.";
        }
      } else {
        if ($idlancdtmai == 'S') {
          if ( $dtretirada > ($dtprocessa->add(new \DateInterval('P'.$nrdiassaidpd.'D')))) {
            $mensagem = "Data informada fora do período permitido.";
          }
        } else {
          $mensagem = "Data informada não deve ser maior que a data de processamento.";
        }
      }
    }
    return $mensagem;
  }
  
  private function retornaRetirada($row) {
    $nrcardapio = $this->service->retornaNrcardapio($row);
    return $this->service->retornaRetiradasExistentes($row['CDFILIAL'], $nrcardapio);
  }

  public function updateStock(Request\Row $request, Response $response) {
    try {
      $row = $request->getRow();
      $cdfilancestq = $this->service->retornaFilialLancamentoRetirada($row['CDFILIMOVI'], $row['NRCARDAPIO']);
      $this->service->atualizaEstoque($cdfilancestq, $row['NRLANCESTQ']);
    } catch (\Exception $e) {
      $response->setError(new DTO\Response\Error($e->getMessage(), 1));
    }
  }

  public function deleteProduct(Request\DataSet $request, Response $response) {
    try {
      $this->service->beginTransaction();
      $itens = $request->getDataSet()->getRows();
      $firstItem = null;
      if(count($itens) > 0){
        $firstItem = $itens[0];
      }
      $cdfilancestq = $this->service->retornaFilialLancamentoRetirada($firstItem['CDFILIAL'], $firstItem['NRCARDAPIO']);
      foreach ($itens as $item) {
        $this->service->excluiItemreti($item);
        if (isset($item['NRLANCESTQ']) && $item['NRLANCESTQ']) {
          $item['CDFILANCESTQ'] = $cdfilancestq;
          $this->service->excluiItemLancto($item);
        }
      }
      if($firstItem){
          $this->service->atualizaEstoque($cdfilancestq, $firstItem['NRLANCESTQ']); // passava idevolestoq como 'S'
      }
      $this->service->commit();
    } catch (\Exception $e) {
      $this->service->rollback();
      $this->treatError($e, $response);
    }
  }
  
  public function deleteWaybill(Request\Row $request, Response $response) {
    try {
      $row = $request->getRow();
      $this->service->excluiLancamentoRetirada($row);
    } catch (\Exception $e) {
      $this->treatError($e, $response);
    }
  }
  
  private function setParamsEstoq(&$row){
    $row['CDALMOXARIFE'] = isset($row['CDALMOXARIFE']) && $row['CDALMOXARIFE'] ? $row['CDALMOXARIFE'] : ' '; 
    $row['CDLOCALESTOQ'] = isset($row['CDLOCALESTOQ']) && $row['CDLOCALESTOQ'] ? $row['CDLOCALESTOQ'] : ' '; 
    $row['NRLOTEESTQ']   = isset($row['NRLOTEESTQ'])   && $row['NRLOTEESTQ']   ? $row['NRLOTEESTQ']   : ' '; 
    $row['NRSUBLOTE']    = isset($row['NRSUBLOTE'])    && $row['NRSUBLOTE']    ? $row['NRSUBLOTE']    : ' ';   
  }

  public function updateProduct(Request\Row $request, Response $response) {
    $row = $request->getRow();
    try{
      // $this->service->beginTransaction();
      $cdfilimovi = $row['CDFILIMOVI'];
      $nrlancestq = $row['NRLANCESTQ'];
      
      $error = array();
      $validaEstoque = $this->service->isValidStock($row, $error);

      if(!$validaEstoque) {
        $response->addDataSet(new DataSet('error', $error));
        throw new \Exception("INVALID_STOCK");
      }
      
      $cdfilancestq = $this->service->retornaFilialLancamentoRetirada($row['CDFILIAL'], $row['NRCARDAPIO']);
      $itlanctoest = $this->service->retornaItlanctoest($cdfilancestq, $nrlancestq, $row['NRSEQUITRETI']);

      $row['CDFILANCESTQ'] = $cdfilancestq;
      if($row['QTRETIRADA'] > 0) {
        if($itlanctoest) {
          $rowItemLancto = clone $row;
          $rowItemLancto['CDFILIAL'] = $cdfilancestq;
          $this->service->alteraItemLancto($rowItemLancto);
        } else {
          $row['NRSEQUITEM'] = $row['NRSEQUITRETI'];
          $row['IDTIPOMOVI'] = '3';
          $row['IDORDEMCLC'] = 9;
          $row['IDEVOLESTOQ'] = 'N';
          $this->setParamsEstoq($row);
          $this->service->insereItemLancto($row, $cdfilancestq);
        }
      } else {
        if($itlanctoest) {
          $this->service->removeRelacaoItemRetiItlanctoest($row);
          $this->service->excluiItemLancto($row);
        }
      }
      
      $this->service->alteraItemreti($row);
      $this->service->atualizaEstoque($cdfilancestq, $nrlancestq);
        
      // $this->service->commit();
    } catch (\Exception $e) {
      // $this->service->rollback();
      $this->treatError($e, $response);
    }
  }
  
  public function saveNewWithdrawal(Request\Row $request, Response $response) {
    try {
      $row = $request->getRow();
      $dtretirada = $row['DTRETIRADA'];
      
    //   //coleta de dados
      $this->libUtilities->saveCollectedData($row['CDFILIAL'],'RETIRADA');
      
      $this->setParamsEstoq($row);
      $dtposicao = $this->service->getDataDeString($dtretirada);
      $dtposicao->add(new \DateInterval('P1D'));
      $dtposicao = $dtposicao->format('d/m/Y');
      $cdfilial_logada = $this->environment->getCdfilial();
      $row['CDFILILANC'] = null;
      $row['NRLANCESTQ'] = null;
      $row['NRSEQUITEM'] = '0000';
      $row['CDPRODMOVI'] = null;
      $row['DTPOSICAO'] = $dtposicao;
      $this->service->beginTransaction();  
      try {
        $error = array();
        $isValidStock = $this->service->isValidStock($row, $error);
        if(!$isValidStock) {
          $response->addDataSet(new DataSet('error', $error));
          throw new \Exception("INVALID_STOCK");
        }
        $nrcardapio = $this->service->retornaNrcardapio($row);
        if(!$nrcardapio) {
          $nrcardapio = $this->service->insereCardapio($row);
        }
        $row['IDTPLANCTO'] = 3;
        $row['DTLANCESTQ'] = $dtretirada;
        $row['DSLANCESTQ'] = "Retirada para o cardápio: ".$dtretirada.", Servico: ".$row['CDSERVFILI'];
        
        $nrlancto = $this->service->insereLancamento($row);
        $row['NRCARDAPIO'] = $nrcardapio;
        $row['NRLANCESTQ'] = $nrlancto;
        $nrretirada = $this->service->insereRetirada($row);
        
        $row['IDTIPOMOVI'] = '3';
        $row['IDORDEMCLC'] = 9;
        $row['IDEVOLESTOQ'] = 'N';
        $row['NRLANCESTQ'] = $nrlancto;
        $row['NRRETIRADA'] = $nrretirada;
        $row['NRCARDAPIO'] = $nrcardapio;
        $row['IDINTESIST'] = 'N';
        $this->service->insereItemLancto($row, $cdfilial_logada);
        $this->service->insereItemreti($row, $cdfilial_logada);
        $this->service->commit();
      } catch(\Exception $e){
        $this->service->rollback();
        throw $e;
      }
      $this->service->atualizaEstoque($this->environment->getCdfilial(), $nrlancto); // passava idevolestoq como 'S'
    } catch (\Exception $e) {
        $this->treatError($e, $response);
    }
  }
  
  public function saveNewProductOnWithdrawal(Request\Row $request, Response $response) {
    try {
      $row = $request->getRow();
      //coleta de dados
      $this->libUtilities->saveCollectedData($row['CDFILIAL'],'RETIRADA');
      
      $cdfilimovi = $row['CDFILIMOVI'];
      $nrlancestq = $row['NRLANCESTQ'];
      $dtretirada = $row['DTRETIRADA'];
      
      $filialLancamento = $this->service->retornaFilialLancamentoRetirada($cdfilimovi, $row['NRCARDAPIO']);
      $dtposicao = $this->service->getDataDeString($dtretirada);
      $dtposicao->add(new \DateInterval('P1D'));
      $dtposicao = $dtposicao->format('d/m/Y');
      $itemreti = $this->service->Vd_ExistProdAlmox($row);

      if($itemreti) {
        throw new \Exception("Já existe item cadastrado.");
      }

      $maiorseq = $this->service->retornaMaxSequencial($row);
      $seq = str_pad(($maiorseq+1), 4, "0", STR_PAD_LEFT);

      $row['CDFILILANC'] = $filialLancamento;
      $row['NRSEQUITEM'] = $seq;
      $row['CDPRODMOVI'] = null;
      $row['DTPOSICAO'] = $dtposicao;
      $this->setParamsEstoq($row);
      $quantidadeInvalida = $this->service->validaQuantidadeEstoque($row);

      if($quantidadeInvalida) {
        throw new \Exception($quantidadeInvalida);
      }

      $controleEstoque = $this->verificaControleEstoque($row);

      if (!$controleEstoque) {
        throw new \Exception("Produto informado não controla estoque.");
      }
      
      $row['IDTIPOMOVI'] = '3';
      $row['IDORDEMCLC'] = 9;
      $row['IDEVOLESTOQ'] = 'N';
      $row['IDINTESIST'] = 'N';
      // transaçao abaixo nao existia antes, caso ocorra algum erro nesta rotina, remover a transaçao
      try {
        $this->service->beginTransaction();
        $this->service->insereItemLancto($row, $filialLancamento);
        $this->service->insereItemreti($row, $filialLancamento);
        $this->service->commit();
      } catch (\Exception $e) {
        $this->service->rollback();
        $this->treatError($e, $response);
      }
      $this->service->atualizaEstoque($filialLancamento, $nrlancestq);// passava idevolestoq como 'S'

    } catch (\Exception $e) {
      $this->treatError($e, $response);
    }
  }
  
  private function verificaControleEstoque($row) {
    $conditions = array (
      array(
          "columnName" => "CDFILIAL",
          "operator" => "=",
          "value" => $row['CDFILIAL']
      ),
      array(
          "columnName" => "CDPRODUTO",
          "operator" => "=",
          "value" => $row['CDPRODUTO']
      ),
      array(
          "columnName" => "NRORG",
          "operator" => "=",
          "value" => $this->environment->getNrOrg()
      )
    );
    $dataSet = $this->crudService->getFilteredDataSet('prodfili', $conditions);
    $rows = $dataSet->getRows();

    if ($rows[0]['IDCNTRESTOQ'] == 'N')
      return false;
    return true;
  }
}
