import {
Metadata,
Loader,
Http,
I18n,
} from '@zeedhi/core';
import {
Modal,
ModalService,
Select,
Form,
TextInput,
IterableComponentRender,
} from '@zeedhi/common';
import {} from '@zeedhi/teknisa-components-common';

export class AddProductModal {
  private Utilities = Loader.getInstance('Utilities');

  private PlanningWithdrawalController = Loader.getInstance('PlanningWithdrawalController');

  private WaybillLaunchController = Loader.getInstance('WaybillLaunchController');

  private basicModal!: Modal;

  public async createBasicUsageModal() {
    this.basicModal = await ModalService.createFromJson('planning/modals/addProductModal', true);
  }

  public destroyBasicUsageModal() {
    this.basicModal?.destroy();
  }

  public openBasicModal() {
    this.basicModal.show();
  }

  public closeBasicModal() {
    this.basicModal.hide();
  }

  public newProductAdded: string = 'Novo produto adicionado com sucesso.';

  public saveNewProductOnWithdrawal() {
    const form = Metadata.getInstance<Form>('addNewProductForm');
    const withdrawalData = Metadata.getInstance<IterableComponentRender>('detailCardWaybills');
    const disheType = Metadata.getInstance<Select>('disheTypeNewProduct');
    const dishes = Metadata.getInstance<Select>('selectDisheNewProduct');
    let conditions = withdrawalData.datasource.data[0];
    switch (withdrawalData.datasource.data[0].IDRETIPRAT) {
      case 'N':
        disheType.clearValidations();
        dishes.clearValidations();
        break;
      case 'S':
        disheType.addValidation('required');
        dishes.addValidation('required');
        break;
      default:
        break;
    }
    setTimeout(() => {
      const validation = form.validate();
      if (validation) {
        if (conditions.IDRETIPRAT === 'S') {
          const dishe = Metadata.getInstance<Select>('selectDisheNewProduct').value.split('+');
          const [CDPRATO, CDFILAUXPRAT] = dishe;
          conditions.CDTIPOPRAT = Metadata.getInstance<Select>('disheTypeNewProduct').value;
          conditions.CDPRATO = CDPRATO;
          conditions.CDFILAUXPRAT = CDFILAUXPRAT;
        } else {
          conditions.CDTIPOPRAT = '';
          conditions.CDPRATO = '';
          conditions.CDFILAUXPRAT = '';
        }
        conditions.CDPRODUTO = Metadata.getInstance<Select>('selectProductNewProduct').value;
        conditions.DTRETIRADA = conditions.DTLANCESTQ.substring(0, 10);
        conditions.QTPREVISRETI = Metadata.getInstance<Select>('qtPredictedNewProduct').value;
        conditions.QTEXTRARETI = Metadata.getInstance<Select>('qtExtraNewProduct').value;
        conditions.QTDEVOLVRETI = Metadata.getInstance<Select>('qtReturnedNewProduct').value;
        conditions.QTRETIRADA = Metadata.getInstance<Select>('qtConsumedNewProduct').value;
        conditions.CDTIPORETI = Metadata.getInstance<Select>('withdrawalTypeNewProduct').value;
        conditions = { ...this.PlanningWithdrawalController.unitParams, ...conditions };

        Http.post('/saveNewProductOnWithdrawal', {
          requestType: 'Row',
          row: conditions,
        }).then((response) => {
          if (response.data.error) {
            this.Utilities.failMessage(I18n.translate(response.data.error));
          } else {
            const newProductAdded = I18n.translate(this.newProductAdded);
            this.Utilities.successMessage(newProductAdded);
            this.reloadDataSources();
          }
        });
      }
    }, 0);
  }

  public async productsDataSource({ component }: any) {
    await this.openBasicModal();

    const disheType = Metadata.getInstance<Select>('disheTypeNewProduct');
    const dishes = Metadata.getInstance<Select>('selectDisheNewProduct');
    switch (component.name) {
      case 'addProductButton':
        disheType.isVisible = false;
        dishes.isVisible = false;
        break;
      case 'addProductInDisheButton':
        disheType.isVisible = true;
        dishes.isVisible = true;
        break;
      default:
        break;
    }

    const product = Metadata.getInstance<Select>('selectProductNewProduct');
    const branch = Metadata.getInstance<Select>('unitsForWithdrawal');
    const conditions = {
      NRORG: localStorage.getItem('NRORG'),
      CDNVPRODUTO: localStorage.getItem('CDNVPRODUTO'),
      CDFILIAL: branch.value.CDFILIAL,
    };

    this.populateWithdrawalType();
    product.datasource.filter = conditions;
    product.datasource.get();
  }

  public populateSgUnityField({ component }: any) {
    const unity = Metadata.getInstance<TextInput>('productUnityAddNewProduct');
    component.datasource.data.forEach((item: any) => {
      if (item.CDPRODUTO === component.value) {
        unity.value = item.SGUNIDADE;
      }
    });
  }

  public populateWithdrawalType() {
    const withType = this.PlanningWithdrawalController.unitParams.CDTPRETIPADR;
    const withdrawalType = Metadata.getInstance<Select>('withdrawalTypeNewProduct');
    withdrawalType.value = withType;
  }

  public reloadDataSources() {
    const dishes = Metadata.getInstance<IterableComponentRender>('iterableCollapseCardDishesTypes');
    const products = Metadata.getInstance<IterableComponentRender>('iterableProducts');
    if (this.WaybillLaunchController.productsTitleVisibility === true) {
      products.reload();
    }
    if (this.WaybillLaunchController.dishesTypesTitleVisibility === true) {
      dishes.reload();
    }
  }
}
