import {
Metadata,
Loader,
Http,
I18n,
} from '@zeedhi/core';
import {
Modal,
ModalService,
Select,
Form,
TextInput,
// IterableComponentRender,
DialogService,
Date as DateInput,
DateRange,
} from '@zeedhi/common';
import {} from '@zeedhi/teknisa-components-common';

export class AddWaybillModalController {
  private PlanningWithdrawalController = Loader.getInstance('PlanningWithdrawalController');

  private Utilities = Loader.getInstance('Utilities');

  private addWaybillModal!: Modal;

  public async createBasicUsageModal() {
    this.addWaybillModal = await ModalService.createFromJson('planning/modals/addWaybillModal', true);
  }

  public destroyBasicUsageModal() {
    this.addWaybillModal?.destroy();
  }

  public openBasicModal() {
    this.addWaybillModal.show();
  }

  public closeBasicModal() {
    this.addWaybillModal.hide();
  }

  public paramsForNewWithdrawal: any = {
    CDSERVFILI: '',
  };

  public disheTypeVisibility: boolean = true;

  public selectRequestVisibility: boolean = true;

  public addProductVisibility: boolean = false;

  public notEnoughStock: string = 'A confirmação da operação faria com que a posição de estoque de alguns itens se tornasse irregular.';

  public notEnoughStockWarn: string = 'Retirada incluida com sucesso, porém ocorreram erros durante a evolução do estoque, verifique a retirada.';

  public successfullyWithdrawal: string = 'Retirada incluída com sucesso.';

  public addProductTitle: string = 'Adicionar produto?';

  public addProductText: string = 'Retirada não possui produtos, deseja adicionar?';

  public closeConfirmationDialog: any = {
    click: () => DialogService.hide(),
  };

  public filterUnitsByOperator() {
    const unitSelect = Metadata.getInstance<Select>('waybillBranch');
    const conditions = {
      NRORG: localStorage.getItem('NRORG'),
      CDOPERADOR: localStorage.getItem('CDOPERADOR'),
    };

    unitSelect.datasource.filter = conditions;
    unitSelect.datasource.get().then(() => {
      unitSelect.value = this.PlanningWithdrawalController.unitSelected;
    });
  }

  public filterServicesByUnit() {
    const unitSelectValue = Metadata.getInstance<Select>('waybillBranch').value;
    const servicesSelect = Metadata.getInstance<Select>('selectServicesWaybill');

    if (unitSelectValue !== null) {
      const conditions = {
        CDFILIAL: unitSelectValue.CDFILIAL,
      };
      servicesSelect.datasource.filter = conditions;
      servicesSelect.datasource.get();
    } else {
      servicesSelect.datasource.clearFilter();
    }
    this.loadRequests();
  }

  public populateWithdrawalFields({ component }: any) {
    const service = Metadata.getInstance<Select>('selectServicesWaybill');
    const withdrawalDate = Metadata.getInstance<DateInput>('dateWithdrawal');
    const menuDate = Metadata.getInstance<DateInput>('dateMenu');
    if (component.value !== null) {
      service.value = component.selectValue.CDSERVFILI;
      withdrawalDate.value = component.selectValue.DTCARDAPIO;
      menuDate.value = component.selectValue.DTCARDAPIO;
    }
  }

  public loadRequests() {
    const requestsSelect = Metadata.getInstance<Select>('selectRequest');
    const dtprocessa = this.PlanningWithdrawalController.unitParams.DTPROCESSA;

    const conditions = {
      CDFILIAL: this.PlanningWithdrawalController.unitParams.CDFILIAL,
      DTPROCESSA: dtprocessa,
      NRORG: localStorage.getItem('NRORG'),
    };
    requestsSelect.datasource.filter = conditions;
    requestsSelect.datasource.get();
  }

  public newProductWithdrawalVisibility() {
    this.selectRequestVisibility = false;
    this.addProductVisibility = !this.selectRequestVisibility;
    DialogService.hide();
  }

  public newProductsOnWithdrawal = {
    click: () => this.newProductWithdrawalVisibility(),
  };

  public nextStepButtonNewWithdrawal() {
    const requestNumber = Metadata.getInstance<Select>('selectRequest');
    const conditions = this.PlanningWithdrawalController.unitParams;
    // const withdrawalsCards = Metadata.getInstance<IterableComponentRender>('iterableWaybillCard');
    const service = Metadata.getInstance<Select>('selectServicesWaybill');
    const form = Metadata.getInstance<Form>('selectRequestForm');
    conditions.CDSERVFILI = service.value;
    service.datasource.data.forEach((item) => {
      if (item.CDSERVFILI === service.value) {
        conditions.IDUTRETIPRAT = item.IDUTRETIPRAT;
      }
    });
    conditions.DTRETIRADA = Metadata.getInstance<Select>('dateWithdrawal').value;
    conditions.DTCARDAPIO = Metadata.getInstance<Select>('dateWithdrawal').value;
    conditions.CDFILIMOVI = Metadata.getInstance<Select>('unitsForWithdrawal').value.CDFILIAL;
    if (requestNumber.value != null) {
      conditions.NRREQUEXTR = Metadata.getInstance<Select>('selectRequest').value.substring(0, 10);
      conditions.NRREQUEXTRRE = Metadata.getInstance<Select>('selectRequest').value.substring(0, 10);
    } else {
      conditions.NRREQUEXTR = '0000000000';
      conditions.NRREQUEXTRRE = '0000000000';
    }

    const validation = form.validate();
    if (validation) {
      Http.post('/saveWithdrawal', {
        requestType: 'Row',
        row: conditions,
      }).then((response) => {
        if (response.data.error) {
          this.Utilities.failMessage(I18n.translate(response.data.error));
        } else if (response.data.messages) {
          if (response.data.messages[0].message === false) {
            this.paramsForNewWithdrawal = conditions;
            const dialogParams = {
              addProductTitle: I18n.translate(this.addProductTitle),
              addProductText: I18n.translate(this.addProductText),
            };
            this.Utilities.showConfirmationDialog(dialogParams.addProductTitle, dialogParams.addProductText, this.closeConfirmationDialog, this.newProductsOnWithdrawal);
            if (conditions.IDUTRETIPRAT === 'N') {
              this.disheTypeVisibility = false;
              this.productsDataSource();
              this.populateWithdrawalType();
              this.filterWithdrawalAdded();
            } else {
              this.disheTypeVisibility = true;
              this.dishesDataSource();
              this.productsDataSource();
              this.populateWithdrawalType();
              this.filterWithdrawalAdded();
            }
          } else {
            const successfullyWithdrawal = I18n.translate(this.successfullyWithdrawal);
            this.Utilities.successMessage(successfullyWithdrawal);
            // withdrawalsCards.reload();
            this.filterWithdrawalAdded();
            this.closeBasicModal();
          }
        }
      });
    }
  }

  public addNewWithdrawal() {
    const form = Metadata.getInstance<Form>('addProductToWithdrawal');
    const disheType = Metadata.getInstance<Select>('disheType');
    const dishes = Metadata.getInstance<Select>('selectDishe');

    switch (this.paramsForNewWithdrawal.IDUTRETIPRAT) {
      case 'N':
        disheType.clearValidations();
        dishes.clearValidations();
        break;
      case 'S':
        disheType.addValidation('required');
        dishes.addValidation('required');
        break;
      default:
        break;
    }

    setTimeout(() => {
      const validation = form.validate();
      if (validation) {
        // const withdrawalsCards = Metadata.getInstance<IterableComponentRender>('iterableWaybillCard');
        if (this.paramsForNewWithdrawal.IDUTRETIPRAT === 'S') {
          const dishe = Metadata.getInstance<Select>('selectDishe').value.split('+');
          const CDPRATO = dishe[0];
          const CDFILAUXPRAT = dishe[1];
          this.paramsForNewWithdrawal.CDTIPOPRAT = Metadata.getInstance<Select>('disheType').value;
          this.paramsForNewWithdrawal.CDPRATO = CDPRATO;
          this.paramsForNewWithdrawal.CDFILAUXPRAT = CDFILAUXPRAT;
        } else {
          this.paramsForNewWithdrawal.CDPRATO = '';
          this.paramsForNewWithdrawal.CDTIPOPRAT = '';
          this.paramsForNewWithdrawal.CDFILAUXPRAT = '';
        }
        this.paramsForNewWithdrawal.CDPRODUTO = Metadata.getInstance<Select>('selectProduct').value;
        this.paramsForNewWithdrawal.CDTIPORETI = Metadata.getInstance<Select>('withdrawalType').value;
        this.paramsForNewWithdrawal.QTPREVISRETI = Metadata.getInstance<Select>('qtPredicted').value;
        this.paramsForNewWithdrawal.QTEXTRARETI = Metadata.getInstance<Select>('qtExtra').value;
        this.paramsForNewWithdrawal.QTDEVOLVRETI = Metadata.getInstance<Select>('qtReturned').value;
        this.paramsForNewWithdrawal.QTRETIRADA = Metadata.getInstance<Select>('qtConsumed').value;
        Http.post('/saveNewWithdrawal', {
          requestType: 'Row',
          row: this.paramsForNewWithdrawal,
        }).then((response) => {
          if (response.data.error === this.notEnoughStock) {
            const notEnoughStockWarn = I18n.translate(this.notEnoughStockWarn);
            this.Utilities.warnMessage(notEnoughStockWarn);
            this.closeBasicModal();
            this.stepOneNewWithdrawal();
            this.filterWithdrawalAdded();
          } else {
            const successfullyWithdrawal = I18n.translate(this.successfullyWithdrawal);
            this.Utilities.successMessage(successfullyWithdrawal);
            this.closeBasicModal();
            this.stepOneNewWithdrawal();
            this.filterWithdrawalAdded();
          }
        });
      }
    }, 0);
  }

  public async newWithdrawalModal() {
    await this.openBasicModal();

    const unit = Metadata.getInstance<Select>('waybillBranch');
    unit.value = this.PlanningWithdrawalController.unitSelected;
  }

  public stepOneNewWithdrawal() {
    this.addProductVisibility = false;
    this.selectRequestVisibility = !this.addProductVisibility;
  }

  public addWaybill() {
    const service = Metadata.getInstance<Select>('selectServicesWaybill');
    const dateWithdrawal = Metadata.getInstance<DateInput>('dateWithdrawal');
    const dateMenu = Metadata.getInstance<DateInput>('dateMenu');

    const conditions = {
      CDSERVFILI: service.value,
      DTRETIRADA: dateWithdrawal.value,
      DTCARDAPIO: dateMenu.value,
      CDTPRETIPADR: this.PlanningWithdrawalController.unitParams.CDTPRETIPADR,
      CDALMOXARIFE: this.PlanningWithdrawalController.unitParams.CDALMOXARIFE,
      CDLOCALESTOQ: this.PlanningWithdrawalController.unitParams.CDLOCALESTOQ,
      DTPROCESSA: this.PlanningWithdrawalController.unitParams.DTPROCESSA,
      IDLANCDTMEN: this.PlanningWithdrawalController.unitParams.IDLANCDTMEN,
      IDLANCDTMAI: this.PlanningWithdrawalController.unitParams.IDLANCDTMAI,
      NRDIASSAID: this.PlanningWithdrawalController.unitParams.NRDIASSAID,
      NRDIASSAIDPD: this.PlanningWithdrawalController.unitParams.NRDIASSAIDPD,
      IDLANCDTMENFIL: this.PlanningWithdrawalController.unitParams.IDLANCDTMENFIL,
      NRDIASSAIDFIL: this.PlanningWithdrawalController.unitParams.NRDIASSAIDFIL,
      IDLANCDTMAIFIL: this.PlanningWithdrawalController.unitParams.IDLANCDTMAIFIL,
      NRDIASSAIDPDFIL: this.PlanningWithdrawalController.unitParams.NRDIASSAIDPDFIL,
      CDFILIAL: this.PlanningWithdrawalController.unitParams.CDFILIAL,
    };

    Http.post('/addWaybill', {
      requestType: 'Row',
      row: conditions,
    });
  }

  public resetRequestField() {
    const request = Metadata.getInstance<Select>('selectRequest');
    request.value = null;
  }

  public dishesDataSource() {
    const branch = Metadata.getInstance<Select>('waybillBranch');
    const dishesSelect = Metadata.getInstance<Select>('selectDishe');
    const conditions = {
      NRORG: localStorage.getItem('NRORG'),
      CDFILAUXPRAT: branch.value.CDFILIAL,
    };

    dishesSelect.datasource.filter = conditions;
    dishesSelect.datasource.get();
  }

  public productsDataSource() {
    const product = Metadata.getInstance<Select>('selectProduct');
    const branch = Metadata.getInstance<Select>('waybillBranch');
    const conditions = {
      NRORG: localStorage.getItem('NRORG'),
      CDNVPRODUTO: localStorage.getItem('CDNVPRODUTO'),
      CDFILIAL: branch.value.CDFILIAL,
    };

    product.datasource.filter = conditions;
    product.datasource.get();
  }

  public populateSgUnityField({ component }: any) {
    const unity = Metadata.getInstance<TextInput>('productUnityAdd');
    component.datasource.data.forEach((item: any) => {
      if (item.CDPRODUTO === component.value) {
        unity.value = item.SGUNIDADE;
      }
    });
  }

  public populateWithdrawalType() {
    const withType = this.PlanningWithdrawalController.unitParams.CDTPRETIPADR;
    const withdrawalType = Metadata.getInstance<Select>('withdrawalType');
    withdrawalType.value = withType;
  }

  public fillExtra({ component }: any) {
    const extra = Metadata.getInstance<Select>('qtExtra');
    extra.value = component.value;
  }

  public filterWithdrawalAdded() {
    const service = Metadata.getInstance<Select>('selectServicesWaybill').value;
    const dtWithdrawal = Metadata.getInstance<DateInput>('dateWithdrawal').value;
    const dtMenu = Metadata.getInstance<DateInput>('dateMenu').value;

    const serviceSearch = Metadata.getInstance<Select>('unitServices');
    const dtWithdrawalSearch = Metadata.getInstance<DateRange>('dateRangeForWithdrawal');
    const dtMenuSearch = Metadata.getInstance<DateRange>('dateRangeForMenu');
    serviceSearch.value = service;
    dtWithdrawalSearch.value = [dtWithdrawal, dtWithdrawal];
    dtMenuSearch.value = [dtMenu, dtMenu];
    this.PlanningWithdrawalController.loadWaybillData();
  }

  public requestFieldCheckLanguage({ component }: any) {
    const condition = localStorage.getItem('LOGIN_LANGUAGE') === 'en_us';
    if (condition) {
      component.dataText = ['NRREQUEXTR', 'DTCARDAPIOENG', 'CDSERVFILI', 'NMSERVICO'];
    }
  }

  public setWithdrawalDate({ component }: any) {
    this.resetRequestField();
    const withdrawalDate = Metadata.getInstance<DateInput>('dateMenu');
    withdrawalDate.value = component.value;
  }
}
